@@ -13,6 +13,7 @@ with qw(
   GraphQL::Role::Leaf
   GraphQL::Role::Nullable
   GraphQL::Role::Named
+  GraphQL::Role::FieldDeprecation
 );
 
 our $VERSION = '0.02';
@@ -48,7 +49,8 @@ the value. Integers are often useful.
 
 =item deprecation_reason
 
-Reason if deprecated.
+Reason if deprecated. If supplied, the hash for that value will also
+have a key C<is_deprecated> with a true value.
 
 =item description
 
@@ -71,6 +73,23 @@ has values => (
   required => 1,
 );
 
+=head1 METHODS
+
+=head2 BUILD
+
+Internal method.
+
+=cut
+
+sub BUILD {
+  my ($self, $args) = @_;
+  $self->_fields_deprecation_apply('values');
+  my $v = $self->values;
+  for my $name (keys %$v) {
+    $v->{$name}{value} = $name if !exists $v->{$name}{value}; # undef valid
+  }
+}
+
 __PACKAGE__->meta->make_immutable();
 
 1;
