@@ -5,7 +5,7 @@ use strict;
 use warnings;
 use Type::Library
   -base,
-  -declare => qw( StrNameValid FieldMapInput FieldMapOutput Int32Signed );
+  -declare => qw( StrNameValid FieldMapInput ValuesMatchTypes );
 use Type::Utils -all;
 use Types::TypeTiny -all;
 use Types::Standard -all;
@@ -36,10 +36,51 @@ an exception. Suitable for passing to an C<isa> constraint in L<Moo>.
 
 declare "StrNameValid", as StrMatch[ qr/^[_a-zA-Z][_a-zA-Z0-9]*$/ ];
 
+=head2 ValuesMatchTypes
+
+Subtype of L<Types::Standard/HashRef>, whose values are hash-refs. Takes
+two parameters:
+
+=over
+
+=item value keyname
+
+Optional within the second-level hashes.
+
+=item type keyname
+
+Values will be a L<GraphQL::Type>. Mandatory within the second-level hashes.
+
+=back
+
+In the second-level hashes, the values (if given) must pass the GraphQL
+type constraint.
+
+=cut
+
+declare "ValuesMatchTypes",
+  constraint_generator => sub {
+    my ($value_key, $type_key) = @_;
+    declare as HashRef[Dict[
+      $type_key => ConsumerOf['GraphQL::Role::Input'],
+      slurpy Any,
+    ]], where {
+      !grep {
+        $_->{$value_key} and !$_->{$type_key}->is_valid($_->{$value_key})
+      } values %$_
+    }, inline_as {
+      (undef, <<EOF);
+        !grep {
+          \$_->{$value_key} and !\$_->{$type_key}->is_valid(\$_->{$value_key})
+        } values %{$_[1]}
+EOF
+    };
+};
+
 =head2 FieldMapInput
 
-Hash-ref mapping field names to a hash-ref description. Description keys,
-all optional except C<type>:
+Hash-ref mapping field names to a hash-ref
+description. Description keys, all optional except C<type>:
 
 =over
 
@@ -50,7 +91,7 @@ GraphQL input type for the field.
 =item default_value
 
 Default value for this argument if none supplied. Must be same type as
-the C<type>.
+the C<type> (implemented with type L</ValuesMatchTypes>.
 
 =item description
 
@@ -64,17 +105,15 @@ declare "FieldMapInput", as Map[
   StrNameValid,
   Dict[
     type => ConsumerOf['GraphQL::Role::Input'],
-    # TODO: change Any to check that is same as supplied "type". Possibly
-    # with builder?
     default_value => Optional[Any],
     description => Optional[Str],
   ]
-];
+] & ValuesMatchTypes['default_value', 'type' ];
 
 =head2 FieldMapOutput
 
-Hash-ref mapping field names to a hash-ref description. Description keys,
-all optional except C<type>:
+Hash-ref mapping field names to a hash-ref
+description. Description keys, all optional except C<type>:
 
 =over
 
@@ -84,7 +123,7 @@ GraphQL output type for the field.
 
 =item args
 
-Array-ref of L<GraphQL::Argument>s.
+A L</FieldMapInput>.
 
 =item resolve
 
@@ -96,7 +135,8 @@ Code-ref to return a given property from a given source-object.
 
 =item deprecation_reason
 
-Reason if deprecated.
+Reason if deprecated. If given, also sets a boolean key of
+C<is_deprecated> to true.
 
 =item description
 
@@ -110,7 +150,7 @@ declare "FieldMapOutput", as Map[
   StrNameValid,
   Dict[
     type => ConsumerOf['GraphQL::Role::Output'],
-    args => Optional[ArrayRef[InstanceOf['GraphQL::Argument']]],
+    args => Optional[FieldMapInput],
     resolve => Optional[CodeRef],
     subscribe => Optional[CodeRef],
     deprecation_reason => Optional[Str],
@@ -136,15 +176,6 @@ declare "ArrayRefNonEmpty", constraint_generator => sub {
   intersection [ ArrayRef[@_], Tuple[Any, slurpy Any] ]
 };
 
-=head2 Thunk
-
-Can be either a L<Types::TypeTiny/CodeLike> or the type(s) given as
-parameters.
-
-=cut
-
-declare "Thunk", constraint_generator => sub { union [ CodeLike, @_ ] };
-
 =head2 UniqueByProperty
 
 An ArrayRef, its members' property (the one in the parameter) can occur
@@ -168,10 +199,21 @@ declare "UniqueByProperty",
       my %seen;
       !grep $seen{$_->$prop}++, @$_;
     }, inline_as {
-      (undef, "do { my %seen; !grep \$seen{\$_->$prop}++, \@$_[1]; }");
+      (undef, "my %seen; !grep \$seen{\$_->$prop}++, \@{$_[1]};");
     };
   };
 
+=head2 ExpectObject
+
+A C<Maybe[HashRef]> that produces a GraphQL-like message if it fails,
+saying "found not an object".
+
+=cut
+
+declare "ExpectObject",
+  as Maybe[HashRef],
+  message { "found not an object" };
+
 =head1 AUTHOR
 
 Ed J, C<< <etj at cpan.org> >>
